# TemplateSetExporter 2008 M-Logic, Inc.
# This program is distributed under the terms of the
# GNU General Public License, version 2.
#
# $Id: CMS.pm 1409 2008-12-25 06:38:37Z sekine $

package TemplateSetExporter::CMS;

use strict;
use MT::I18N qw( encode_text );

sub start {
    my $app = shift;
    my (%opt) = @_;

    my $q = $app->{query};
    my $blog_id = scalar $q->param('blog_id');

    my $tmpl = $app->load_tmpl('start.tmpl');
    $tmpl->param('blog_id' => $blog_id);
    $app->add_breadcrumb($app->translate("Main Menu"),$app->{mtscript_url});
    require MT::Blog;
    my $blog = MT::Blog->load ($blog_id);
    $app->add_breadcrumb($blog->name,$app->mt_uri( mode => 'menu', args => { blog_id => $blog_id }));
    $app->add_breadcrumb($app->translate("TemplateSetExporter"));

    # Initialize
    $tmpl->param('template_set_module_name'  => 'TemplateSet');
    $tmpl->param('template_set_name' => 'Template set');
    $tmpl->param('template_set_desc' => 'description of template set.');
    $tmpl->param('template_set_author_name' => 'author');
    $tmpl->param('template_set_author_link' => 'http://hoge.com');
    $tmpl->param('template_set_version' => '0.01');

    $tmpl->param('need_name' => $q->param('need_name'));
    $tmpl->param('invalid_name' => $q->param('invalid_name'));
    $tmpl->param(breadcrumbs => $app->{breadcrumbs});
    return $app->build_page($tmpl);
}

sub complete {
    my $app = shift;
    my (%opt) = @_;

    my $q = $app->{query};
    my $blog_id = scalar $q->param('blog_id');

    my $template_set_module_name = $app->param('template_set_module_name');
    unless ($template_set_module_name) {
        $app->add_return_arg(__mode => 'template_set_exporter_start', blog_id => $blog_id, need_name => 1);
        return $app->call_return();
    }
    unless ($template_set_module_name =~ m/\w+/) {
        $app->add_return_arg(__mode => 'template_set_exporter_start', blog_id => $blog_id, invalid_name => 1);
        return $app->call_return();
    }

    my $tmpl = $app->load_tmpl('complete.tmpl');

    $tmpl->param('saved'    => _doExport($app));
    $tmpl->param('set_name' => $template_set_module_name);
    $tmpl->param('blog_id'  => $blog_id);
    $app->add_breadcrumb($app->translate("Main Menu"),$app->{mtscript_url});
    require MT::Blog;
    my $blog = MT::Blog->load ($blog_id);
    $app->add_breadcrumb($blog->name,$app->mt_uri( mode => 'menu', args => { blog_id => $blog_id }));
    $app->add_breadcrumb($app->translate("TemplateSetExporter"));

    $tmpl->param(breadcrumbs       => $app->{breadcrumbs});
    return $app->build_page($tmpl);
}

sub _doExport {
    my $app = shift;

    my $q = $app->{query};
    my $blog_id = scalar $q->param('blog_id');
    my $template_set_module_name = $app->param('template_set_module_name');
    my $dirify_template_set_module_name = MT::Util::dirify($template_set_module_name);

    my $template_set_name = $app->param('template_set_name') || $template_set_module_name;
    my $template_set_desc = $app->param('template_set_desc') || $template_set_module_name;
    my $template_set_author_name = $app->param('template_set_author_name') || 'author';
    my $template_set_author_link = $app->param('template_set_author_link') || 'http://test.com';
    my $template_set_version = $app->param('template_set_version') || '0.01';
    
    require MT::Template;
    require MT::TemplateMap;

    my $blog = MT::Blog->load($blog_id);
    next unless $blog;

    my $set_dir = 'plugins/TemplateSetExporter/templates/' . $dirify_template_set_module_name;
    
    my $cnt = 0;
    while (-e $set_dir) {
        $cnt++;
        $set_dir = 'plugins/TemplateSetExporter/templates/' . $dirify_template_set_module_name . '_' . $cnt;
    }
    use File::Path;
    rmtree($set_dir);
    eval { mkpath($set_dir) };
    return 0 if $@;

    my $templ_dir = $set_dir . '/templates';
    eval { mkpath($templ_dir) };
    return 0 if $@;

    # $module   = "FooBar"
    # $file     = "foo_bar"
    my $module = $template_set_module_name;
    $module =~ s! !!g;
    (my $file = $module) =~ s!([a-z0-9])([A-Z])!$1_$2!g;
    $file =~ s!^_!!g;
    $file = lc $file;

    my $yaml = <<"YAML";
key: MT::Plugin::$module
name: $template_set_name
description: $template_set_desc
author_name: $template_set_author_name
author_link: $template_set_author_link
version: $template_set_version

template_sets:
  $file:
    label: $template_set_name
    base_path: templates
    order: 100
    templates:
YAML

    my $index_yaml =<<"YAML";
      index:
YAML
    my $individual_yaml =<<"YAML";
      individual:
YAML
    my $archive_yaml =<<"YAML";
      archive:
YAML
    my $module_yaml =<<"YAML";
      module:
YAML
    my $widget_yaml =<<"YAML";
      widget:
YAML
    my $widgetset_yaml =<<"YAML";
      widgetset:
YAML
    my $system_yaml =<<"YAML";
      system:
YAML
    
    my $tmpl_iter = MT::Template->load_iter({
        blog_id => $blog->id,
        type => { not => 'backup' },
    });

    return 0 unless $tmpl_iter;

    while (my $tmpl = $tmpl_iter->()) {
        my $name = $tmpl->identifier;
        $name = $tmpl->name unless $name;

        my $dirify_name = MT::Util::dirify($name);
        $dirify_name = 'template' if ($dirify_name eq '') || ($dirify_name =~ m!^_!);
        my $filename = $dirify_name . ".mtml";
        my $file = "$templ_dir/$filename";
        
        my $cnt = 0;
        while (-e $file) {
            $cnt++;
            $filename = $dirify_name . "_$cnt.mtml";
            $file = "$templ_dir/" . $filename;
        }
        
        my $tmpl_text = encode_text($tmpl->text, undef, 'utf-8');
        open FH, ">$file";
        print FH $tmpl_text;
        close FH;

        my $template_name = $filename;
        $template_name =~ s!\.mtml!!;

        my $label = $tmpl->name;
        my $outfile = $tmpl->outfile;
        my $rebuild_me = $tmpl->rebuild_me;

        if ( $tmpl->type =~
            m/^(index)$/ ) {
            #index
    $index_yaml .= <<"YAML";
        $template_name:
          label: $label
          outfile: $outfile
          rebuild_me: $rebuild_me
YAML
        } elsif ( $tmpl->type =~
            m/^(individual|page)$/ ) {
            #individual
            my $maps .= _make_maps($blog->id, $tmpl->id, 'individual');
            $individual_yaml .= <<"YAML";
        $template_name:
          label: $label
$maps
YAML
        } elsif ( $tmpl->type =~
            m/^(archive|category)$/ ) {
            #archive
            my $maps .= _make_maps($blog->id, $tmpl->id, 'archive');
            $archive_yaml .= <<"YAML";
        $template_name:
          label: $label
$maps
YAML
        } elsif ( $tmpl->type =~
            m/^(custom)$/ ) {
            #module
    $module_yaml .= <<"YAML";
        $template_name:
          label: $label
YAML
        } elsif ( $tmpl->type =~
            m/^(widget)$/ ) {
            #widget
    $widget_yaml .= <<"YAML";
        $template_name:
          label: $label
YAML
        } elsif ( $tmpl->type =~
            m/^(widgetset)$/ ) {
            #widget
            $widgetset_yaml .= <<"YAML";
        $template_name:
          label: $label
          order: 1000
YAML
            if ( my $modulesets = $tmpl->modulesets ) {
                my $widgets_yaml .= <<"YAML";
          widgets:
YAML
                foreach my $module ( split ',', $modulesets ) { 
                    my $widget = MT::Template->load($module);
                    my $widget_name = $widget->name;
                    $widgets_yaml .= <<"YAML";
            - $widget_name
YAML
                }
                $widgetset_yaml .= $widgets_yaml;
            }
        } else {
            #system templates
    $system_yaml .= <<"YAML";
        $template_name:
          label: $label
YAML
        }
    }

    $yaml .= $index_yaml;
    $yaml .= $individual_yaml;
    $yaml .= $archive_yaml;
    $yaml .= $module_yaml;
    $yaml .= $widget_yaml;
    $yaml .= $widgetset_yaml;
    $yaml .= $system_yaml;
    $yaml = encode_text($yaml, undef, 'utf-8');

    open FH, ">$set_dir/config.yaml";
    print FH $yaml;
    close FH;

    return 1;
}

sub _make_maps {
    my ($blog_id, $tmpl_id, $type) = @_;
    my $map_iter = MT::TemplateMap->load_iter({
        blog_id => $blog_id,
        template_id => $tmpl_id,
    });

    return '' unless $map_iter;
    
    my $maps .= <<"YAML";
          mappings:
YAML

    while (my $map = $map_iter->()) {
        my $mapid = $map->id;
        $mapid = '_' . $mapid;
        my $archive_type = $map->archive_type;
        my $preferred = $map->is_preferred;
        $maps .= <<"YAML";
            $type$mapid:
              archive_type: $archive_type
              preferred: $preferred
YAML

        my $file_template = $map->file_template;
        if ($file_template) {
            $maps .= <<"YAML";
              file_template: $file_template
YAML
        }
    }

    $maps;
}

1;
